@@ -1,7 +1,7 @@
-"ooolib-python - Copyright (C) 2006 Joseph Colton"
+"ooolib-python - Copyright (C) 2006-2007 Joseph Colton"
 
 # ooolib-python - Python module for creating Open Document Format documents.
-# Copyright (C) 2006  Joseph Colton
+# Copyright (C) 2006-2007  Joseph Colton
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@ import re
 
 def version():
 	"Get the ooolib-python version"
-	return "ooolib-python-0.0.8"
+	return "ooolib-python-0.0.11"
 
 def clean_string(data):
 	"Returns an XML friendly copy of the data string"
@@ -290,7 +290,9 @@ class CalcStyles:
 		self.property_cell_underline = False    # Underline off be default
 		self.property_cell_fg_color = 'default' # Text Color Default
 		self.property_cell_bg_color = 'default' # Cell Background Default
-
+		self.property_cell_fontsize = '10'      # Cell Font Size Default
+		self.property_cell_valign = 'default'	# Vertial Alignment Default
+		self.property_cell_halign = 'default'	# Horizantal Alignment Default
 
 	def get_next_style(self, style):
 		"Returns the next style code for the given style"
@@ -321,6 +323,7 @@ class CalcStyles:
 			if name == 'bold' and type(value) == type(True): self.property_cell_bold = value
 			if name == 'italic' and type(value) == type(True): self.property_cell_italic = value
 			if name == 'underline' and type(value) == type(True): self.property_cell_underline = value
+			if name == 'fontsize': self.property_cell_fontsize = value
 			if name == 'color':
 				self.property_cell_fg_color = 'default'
 				redata = re.search("^(#[\da-fA-F]{6})$", value)
@@ -329,6 +332,10 @@ class CalcStyles:
 				self.property_cell_bg_color = 'default'
 				redata = re.search("^(#[\da-fA-F]{6})$", value)
 				if redata: self.property_cell_bg_color = value.lower()
+			if name == 'valign':
+				self.property_cell_valign = value
+			if name == 'halign':
+				self.property_cell_halign = value
 
 	def get_style_code(self, style):
 		style_code = ""
@@ -360,8 +367,12 @@ class CalcStyles:
 			if self.property_cell_bold: style_data.append(('bold', True))
 			if self.property_cell_italic: style_data.append(('italic', True))
 			if self.property_cell_underline: style_data.append(('underline', True))
+			if self.property_cell_fontsize != '10': style_data.append(('fontsize', self.property_cell_fontsize))
 			if self.property_cell_fg_color != 'default': style_data.append(('color', self.property_cell_fg_color))
 			if self.property_cell_bg_color != 'default': style_data.append(('background', self.property_cell_bg_color))
+			if self.property_cell_valign != 'default': style_data.append(('valign', self.property_cell_valign))
+			if self.property_cell_halign != 'default': style_data.append(('halign', self.property_cell_halign))
+
 			style_data = tuple(style_data)
 			if style_data in self.style_config:
 				# Style Exists, return code
@@ -422,8 +433,37 @@ class CalcStyles:
 					name, value = set
 					if name == 'background':
 						tagline.append(['element', 'fo:background-color', value])
+					if name == 'valign':
+						if value in ['top', 'bottom', 'middle']: 
+							tagline.append(['element', 'style:vertical-align', value])
+					if name == 'halign':
+						tagline.append(['element', 'style:text-align-source', 'fix'])
+						if value in ['filled']:
+							tagline.append(['element', 'style:repeat-content', 'true'])
+						else:
+							tagline.append(['element', 'style:repeat-content', 'false'])
+
 				style_list.append(tagline)
 
+				# Paragraph Properties
+				tagline = ['tagline', 'style:paragraph-properties']
+				tagline_valid = False
+				for set in style_data:
+					name, value = set
+					if name == 'halign':
+						tagline_valid = True
+						if value in ['center']:
+							tagline.append(['element', 'fo:text-align', 'center'])
+						if value in ['end', 'right']:
+							tagline.append(['element', 'fo:text-align', 'end'])
+						if value in ['start', 'filled', 'left']:
+							tagline.append(['element', 'fo:text-align', 'start'])
+						if value in ['justify']:
+							tagline.append(['element', 'fo:text-align', 'justify'])
+				# Conditionally add the tagline
+				if tagline_valid: style_list.append(tagline)
+
+
 				# Text Properties
 				tagline = ['tagline', 'style:text-properties']
 				for set in style_data:
@@ -438,6 +478,8 @@ class CalcStyles:
 						tagline.append(['element', 'style:text-underline-color', 'font-color'])
 					if name == 'color':
 						tagline.append(['element', 'fo:color', value])
+					if name == 'fontsize':
+						tagline.append(['element', 'fo:font-size', '%spt' % value])
 				style_list.append(tagline)
 
 				automatic_styles.append(style_list)
@@ -523,8 +565,13 @@ class CalcSheet:
 			print "Invalid Cell"
 			return
 		# Fix String Data
-		if datatype == 'string':
+		if datatype in ['string', 'annotation']:
 			datavalue = clean_string(datavalue)
+		# Fix Link Data. Link's value is a tuple containing (url, description)
+		if (datatype == 'link'):
+			url = clean_string(datavalue[0])
+			desc = clean_string(datavalue[1])
+			datavalue = (url, desc)
 		# Fix Formula Data
 		if datatype == 'formula':
 			datavalue = self.clean_formula(datavalue)
@@ -532,11 +579,25 @@ class CalcSheet:
 		if col > self.max_col: self.max_col = col
 		if row > self.max_row: self.max_row = row
 		datatype = str(datatype)
-		if (datatype != 'string' and datatype != 'float' and datatype != 'formula'):
+		if (datatype not in ['string', 'float', 'formula', 'annotation', 'link']):
 			# Set all unknown cell types to string
 			datatype = 'string'
-		datavalue = str(datavalue)
-		self.sheet_values[cell] = (datatype, datavalue)
+			datavalue = str(datavalue)
+
+		# The following lines are taken directly from HPS
+		# self.sheet_values[cell] = (datatype, datavalue)
+		# HPS: Cell content is now a list of tuples instead of a tuple
+		# While storing here, store the cell contents first and the annotation next. While generating the XML reverse this
+		contents = self.sheet_values.get(cell, {'annotation':None,'link':None, 'value':None})
+		if datatype == 'annotation':
+			contents['annotation'] = (datatype, datavalue)
+		elif datatype == 'link':
+			contents['link'] = (datatype, datavalue)
+		else:
+			contents['value'] = (datatype, datavalue)
+
+		self.sheet_values[cell] = contents
+
 
 	def get_lists(self):
 		"Returns nested lists for XML processing"
@@ -588,27 +649,50 @@ class CalcSheet:
 					if cell in self.sheet_config:
 						style_code = self.sheet_config[cell] # Lookup cell if available
 					if cell in self.sheet_values:
-						(datatype, datavalue) = self.sheet_values[cell]
+						# (datatype, datavalue) = self.sheet_values[cell] # Marked for removal
 						collist = ['tag', 'table:table-cell']
 						if style_code != 'ce1':
 							collist.append(['element', 'table:style-name', style_code])
-						if datatype == 'float':
-							collist.append(['element', 'office:value-type', datatype])
-							collist.append(['element', 'office:value', datavalue])
-							collist.append(['tag', 'text:p',
-							  ['data', datavalue]])
-						if datatype == 'string':
-							collist.append(['element', 'office:value-type', datatype])
-							collist.append(['tag', 'text:p',
-							  ['data', datavalue]])
-						if datatype == 'formula':
-							# '=SUM(A1:A2)'
-							# datavalue = 'oooc:=SUM([.A1:.A2])'
-							collist.append(['element', 'table:formula', datavalue])
-							collist.append(['element', 'office:value-type', 'float'])
-							collist.append(['element', 'office:value', '0'])
-							collist.append(['tag', 'text:p',
-							  ['data', '0']])
+
+						# Contents, annotations, and links added by HPS
+						contents = self.sheet_values[cell] # cell contents is a dictionary
+						if contents['value']:
+							(datatype, datavalue) = contents['value']
+							if datatype == 'float':
+								collist.append(['element', 'office:value-type', datatype])
+								collist.append(['element', 'office:value', datavalue])
+
+							if datatype == 'string':
+								collist.append(['element', 'office:value-type', datatype])
+							if datatype == 'formula':
+								collist.append(['element', 'table:formula', datavalue])
+								collist.append(['element', 'office:value-type', 'float'])
+								collist.append(['element', 'office:value', '0'])
+								datavalue = '0'
+						else:
+							datavalue = None
+
+						if contents['annotation']:
+							(annotype, annoval) = contents['annotation']
+							collist.append(['tag', 'office:annotation',
+							  ['tag', 'text:p', ['data', annoval]]])
+
+						if contents['link']:
+							(linktype, linkval) = contents['link']
+							if datavalue:
+								collist.append(['tag', 'text:p', ['data', datavalue],
+								  ['tag', 'text:a', ['element', 'xlink:href', linkval[0]],
+								  ['data', linkval[1]]]])
+							else: # no value; just fill the link
+								collist.append(['tag', 'text:p',
+								  ['tag', 'text:a', ['element', 'xlink:href', linkval[0]],
+								  ['data', linkval[1]]]])
+						else:
+							if datavalue:
+								collist.append(['tag', 'text:p', ['data', datavalue]])
+
+
+
 					else:
 						collist = ['tagline', 'table:table-cell']
 					rowlist.append(collist)
